package com.hgny.protobuf;

import com.google.gson.Gson;
import com.google.protobuf.FloatValue;
import com.hgny.protobuf.protobuf.MeterData;
import com.hgny.protobuf.protobuf.MeterDataDay;
import com.hgny.protobuf.test.AddressBook;
import com.hgny.protobuf.test.Person;
import com.hgny.protobuf.test2.AddressBook2;
import com.hgny.protobuf.test2.Person2;
import com.hgny.protobuf.test3.AddressBook3;
import com.hgny.protobuf.test3.Person3;
import com.hgny.protobuf.utils.Constant;
import com.hgny.protobuf.utils.Tools;

import javax.tools.Tool;
import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class Main {
  private static List<String> SKIP_KEY = new ArrayList<>(List.of(new String[]{"mdp", "mdptime", "mdkwh", "mdkwhtime"}));
  public static void main(String[] args) throws IOException, InvocationTargetException, IllegalAccessException {
//    test2();
//    test5();
//    test6();
//    test3();
//    test4();
//    test();
//    testCSV();
//     解压tar.gz
    long start = System.currentTimeMillis();
    for (File gz : new File("./data").listFiles()) {
      if (gz.isDirectory()) {
        continue;
      }
      Tools.extraTarGz(gz, "./data");
    }
    // 遍历目录解压tar.bz2
    for (File file : new File("./data").listFiles()) {
      if (!file.isDirectory()) {
        continue;
      }
      String name = file.getName();
      scanDir(file, "./data/" + name, 0);
    }
    System.out.println("spend: " + (System.currentTimeMillis() - start) + "ms");
    // 压缩
//    for (File file : new File("./data").listFiles()) {
//      if (!file.isDirectory()) {
//        continue;
//      }
//      String name = file.getName();
//      compress(file, "./output/" + name, 0);
//    }
  }

  private static void testCSV() throws IOException {
    File file = new File("./00.csv");
    if (!file.createNewFile()) {
      System.err.println("create file error");
    }
    FileWriter writer = new FileWriter(file);
    writer.write(String.join(",", Constant.KEYS));
    writer.write("\n");
    writer.write("1");
    writer.flush();
    FileReader fileReader = new FileReader("./00.txt");
    BufferedReader bufferedReader = new BufferedReader(fileReader);
    String line = null;
    Gson gson = new Gson();
    while ((line = bufferedReader.readLine()) != null) {
      Map map = gson.fromJson(line, Map.class);
      writer.write(Tools.dataParser(file) + "\n");
      writer.flush();
    }
  }

  private static void scanDir(File dir, String basePath, int level) throws IOException {
    for (File file : dir.listFiles()) {
      String fileName = file.getName();
      if (file.isDirectory()) {
        scanDir(file, basePath + "/" + fileName, level + 1);
      } else {
        Tools.extraTarBz2(file, basePath);
      }
      // 解压后压缩
      if (level == 2) {
        String p = file.getPath().replaceAll("\\\\", "/");
        String output = p.replace("./data/", "./output/");
        compress(file, output, 3);
        // 压缩后删除源文件
        deleteFolder(file);
      }
    }
  }

  private static List<List<String>> compress(File dir, String outputPath, int level) throws IOException {
    List<List<String>> group = new ArrayList<>();
    for (File file : dir.listFiles()) {
      String fileName = file.getName();
      if (file.isDirectory()) {
        List<List<String>> compress = compress(file, outputPath + "/" + fileName, level + 1);
        if (level == 3) {
          group.addAll(compress);
        }
      } else {
        if (!fileName.endsWith(".txt")) {
          continue;
        }
        List<String> strings = Tools.dataParser(file);
        group.add(strings);
      }
    }
    // 读取完整个目录, 开始写入
    if (level == 3) {
      String savePath = outputPath.substring(0, outputPath.substring(0, outputPath.lastIndexOf("/")).lastIndexOf("/"));
      Tools.createDir(savePath, null);
      String s1 = outputPath.replace("./output/", "");
      int index = s1.lastIndexOf("/");
      String s2 = s1.substring(0, index);
      String month = s1.substring(index + 1);
      String fileName = s2.replaceAll("/", "@") + month;
      File file = new File(savePath + "/" + fileName + ".bz2");
      String s = group.stream().map(list -> String.join("\n", list)).collect(Collectors.joining("\n"));
      Tools.compressBZip2(s, file);
    } else if (level == 4) {
      return group;
    }
    return null;
  }

  @SuppressWarnings("ALL")
  public static void test() throws IOException, InvocationTargetException, IllegalAccessException {
    MeterDataDay.Builder builder = MeterDataDay.newBuilder();
    FileReader fileReader = new FileReader("./00.txt");
    BufferedReader bufferedReader = new BufferedReader(fileReader);
    String line = null;
    Gson gson = new Gson();
    while ((line = bufferedReader.readLine()) != null) {
      Map map = gson.fromJson(line, Map.class);
      Map<String, Object> data = (Map<String, Object>) map.get("data");
      String time = (String) map.get("time");
      MeterData meterData = transferData(data, time);
      builder.addMeterData(meterData);
    }
    builder.build().writeTo(new FileOutputStream("./00.dat"));
  }

  public static MeterData transferData(Map<String, Object> map, String time)
      throws IllegalAccessException, IllegalArgumentException, java.lang.reflect.InvocationTargetException {
    MeterData.Builder builder = MeterData.newBuilder();
    map.forEach((key, value) -> {
      if (SKIP_KEY.contains(key)) {
        return;
      }
      if (!(value instanceof Double)) {
        return;
      }
      for (Method method : builder.getClass().getDeclaredMethods()) {
        String methodName = method.getName().toLowerCase();
        if (methodName.contains(key) && methodName.contains("set")) {
          method.setAccessible(true);
          try {
            if (method.getParameterTypes()[0] == float.class) {
              method.invoke(builder, ((Double) value).floatValue());
            } else {
              method.invoke(builder, value);
            }
          } catch (InvocationTargetException | IllegalAccessException e) {
            throw new RuntimeException(e);
          }
        }
      }
    });
    DateTimeFormatter formatter = new DateTimeFormatterBuilder()
        .appendPattern("yyyy-MM-dd HH:mm:ss")
        .toFormatter();
    LocalDateTime parse = LocalDateTime.parse(time, formatter);
    long timestamp = parse.toInstant(ZoneOffset.ofHours(8)).toEpochMilli();
    builder.setTime(Tools.parseTimestamp(timestamp));
    return builder.build();
  }

  private static void deleteFolder(File file) {
    for (File f : file.listFiles()) {
      if (f.isDirectory()) {
        deleteFolder(f);
      } else {
        f.delete();
      }
    }
    file.delete();
  }
}